<?php
/**
 * @version   3.1 
 * @author    Dovi42 
 * @home      http://dovi42.hu/
 * @package   Files library
 * @license   All Rights Reserved - Minden jog fenntartva
 * @copyright (C) 2020 Dovalovszki Tamás - All rights reserved.
 **/

/* 
    Use:
    if (!class_exists ('JDovi42Files', false)) {
       include_once(dirname(__FILE__) .DS. 'jdovi42_files.php');
    }
    $source  = new JDovi42Files('<absolute_path>');
    $folders = $source->getFolders();
    $files   = $source->getFiles();
    
    $folders[0] => Array(
        [name]      => foldername          // folder name
        [title]     => This is test folder // folder title in descript.ion or folder name
        [alias]     => foldername          // foldername alias
        [date]      => 2020-01-01 00:00:00 // date('Y-m-d H:i:s')
        [filename]  => foldername          // folder name
    )

    $files[0] => Array(
        [name]      => filename            // file name
        [title]     => This is test file   // file title in descript.ion or file name
        [alias]     => filename-ext        // filename alias
        [ext]       => ext                 // file extension  
        [size]      => 0                   // file size in bytes
        [size_text] => 0 KB                // formatted file size
        [date]      => 2020-01-01 00:00:00 // date('Y-m-d H:i:s')
        [filename]  => filename.ext        // filename and extension
    )
  
*/

if (!defined('DS')) { define( 'DS', DIRECTORY_SEPARATOR ); }

class JDovi42Files extends stdClass
{
    public $path            = null;

    // Folders properties
    public $folders         = null;
    public $folderSort      = 'sortByList';
    public $folderSortOrder = 'asc';

    // Files properties
    public $files           = array();
    public $allowExt        = array();
    public $filesSort       = 'sortByList';
    public $filesSortOrder  = 'asc';

    // Descriptions properties
    private $descriptionFile = 'descript.ion';
    private $descriptions = array();

    public  $tilt     = array('index.htm', 'index.html', 'descript.ion', '.', '..', '.htaccess', 'filesort.txt');
    private $sortMode = null;
    private $sortList;

    
	public function __construct($path = null, $descriptionFile = '')
	{
        if ($path != null) {
            if (substr($path, -1) == DS) {
                $path = substr($path, 0, -1);
            }
            $this->path = $path;
            
            if (!empty($descriptionFile)) {
                $this->descriptionFile = $descriptionFile;  
                $this->addTilt = $descriptionFile;   
            }

            if (file_exists($this->path .DS. $this->descriptionFile)) {
                $this->getDescriptionsInFile();
            }
        }
    }    

    public function addTilt($list = null)
    {
        if ($list != null) {
            if (is_array($list)) {
                $this->tilt = array_merge($this->tilt, $list);
            } else {
                $this->tilt[] = $list;
            }
        }
    }

    function getFolders()
    {
        if (empty($this->folders)) {
            $folders = array();
            $handle = @opendir($this->path);
            if ($handle) {
                while (false !== ($folder = readdir($handle)))
    			{
                    if ((!in_array($folder, $this->tilt)) and (is_dir($this->path .DS. $folder))) {
                        $folders[] = array('name'     => $folder, 
                                           'title'    => $this->getTitle($folder), 
                                           'date'     => date('Y-m-d H:i:s', filemtime($this->path .DS. $folder)),
                                           'alias'    => $this->getAlias($folder),
                                           'filename' => $folder);
                    }  
                }            
            }
            if ($this->folderSort == 'sortByList') {
                if (file_exists($this->path .DS. 'filesort.txt')) {
                    $this->sortList = file($this->path .DS. 'filesort.txt', FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
                } else {
                    $this->sortList = '';
                    $this->folderSort = 'sortByDate';
                    $this->sortMode = 'desc';
                }
            } else {
                $this->sortMode = $this->folderSortOrder;
            }
            usort($folders, array('JDovi42Files', $this->folderSort));
            $this->folders = $folders;
        }
        return $folders;
    }

    function newFolder($alias, $title)
    {
        if (!file_exists($this->path .DS. $alias)) {
            mkdir($this->path .DS. $alias, 0777);
            file_put_contents($this->path .DS. $alias .DS. 'index.html', '<html><body bgcolor="#FFFFFF"></body></html>');
            // chmod($this->path .DS. $alias .DS. 'index.html', 0664);
            
            $this->descriptions[$alias] = trim($title);
            $this->saveDescriptions();
            
            $item = array(
                    'name'     => $alias, 
                    'title'    => trim($title), 
                    'date'     => date('Y-m-d H:i:s'),
                    'alias'    => $alias,
                    'filename' => $alias
                );
            return $item;
        } 
        return false;
    }
    
    function getEmptyName($name)
    {
        $info = pathinfo($this->path .DS. $name);
        
        $j = 0;
        $tmp = $info['filename'];
        $ext = '';
        if (isset($info['extension'])) {
            $ext = '.' . $info['extension'];
        }
        $filename = $tmp . $ext;
        while (file_exists($this->path .DS. $filename) != false) {
            $j++;
            $filename = $tmp . '-' . $j . $ext;
        }
        return $filename;
    }

    private function recurseRmdir($dir) 
    {
        $files = array_diff(scandir($dir), array('.','..'));
        foreach ($files as $file) {
        (is_dir("$dir/$file")) ? recurseRmdir("$dir/$file") : unlink("$dir/$file");
        }
        return rmdir($dir);
    }

    function deleteFolder($name)
    {
        if (is_dir($this->path .DS. $name)) {
            $this->recurseRmdir($this->path .DS. $name);
            if (isset($this->descriptions[$name])) {
                unset($this->descriptions[$name]);
                $this->saveDescriptions();
            }
            return true;
        } 
        return false;
    }
    
    function rename($oldName, $newName)
    {
        if (empty($oldName) or empty($newName)) {
            return $oldName;
        }
        $newTitle = $newName;
        $newName = $this->getAlias($newName);
        
        $info = pathinfo($this->path .DS. $oldName);
        if (isset($info['extension'])) {
            $newName .= '.' . $info['extension'];
        }
        $newName = $this->getEmptyName($newName, $oldName);
        rename($this->path .DS. $oldName, $this->path .DS. $newName);
        
        $title = '';
        if (isset($this->descriptions[$oldName])) {
            unset($this->descriptions[$oldName]);
        }
        $this->descriptions[$newName] = $newTitle;
        $this->saveDescriptions();

        return $newName;
    }

    function getFiles()
    {
        if (empty($this->files)) {
            $retFiles = null;
            $files = array();
            $handle = @opendir($this->path);
            if ($handle) {
                while (false !== ($file = readdir($handle)))
        		{
                    if ((!in_array($file, $this->tilt)) and (!is_dir($this->path .DS. $file))) {
                        $files[] = $file;     
                    }
                }
            }
            
            if (count($files) > 0) {
                $retFiles = array();
                foreach ($files as $file) {
                
                    $data = pathinfo($this->path .DS. $file);
                    if (!isset($data['extension'])) {
                        $data['extension'] = '';
                    }
                    if ((in_array(strtolower($data['extension']), $this->allowExt)) or (count($this->allowExt) == 0)) {
                        $data['filesize'] = filesize($this->path .DS. $file);
                        $data['filesize_html'] = $this->filesize_format($data['filesize']);
                        $data['created'] = date ("Y-m-d H:i:s", filemtime($this->path .DS. $file));
                        
                        $retFiles[] = array('name'      => $data['filename'], 
                                            'title'     => $this->getTitle($data['basename']), 
                                            'alias'     => $this->getAlias($data['basename']),
                                            'ext'       => strtolower($data['extension']),
                                            'size'      => $data['filesize'],
                                            'size_text' => $data['filesize_html'],
                                            'date'      => $data['created'],
                                            'filename'  => $data['basename']);
                    }
                }  
                if (count($retFiles) > 0) {
    
                    if ($this->filesSort == 'sortByList') {
                        $this->path = str_replace(DS.DS, DS, $this->path);
                        if (file_exists($this->path .DS. 'filesort.txt')) {
                            $this->sortList = file($this->path .DS. 'filesort.txt', FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
                        } else {
                            $this->sortList = '';
                            $this->filesSort = 'sortByName';
                        }
                    }
                    $this->sortMode = $this->filesSortOrder;
                    usort($retFiles, array('JDovi42Files', $this->filesSort));
                }                      
            }
            $this->files = $retFiles; 
        }
        return $this->files;
    }

    function deleteFile($name)
    {
        if (file_exists($this->path .DS. $name)) {
            unlink($this->path .DS. $name);
            if (isset($this->descriptions[$name])) {
                unset($this->descriptions[$name]);
                $this->saveDescriptions();
            }
        }
    }
    
    function sortByDate($a, $b)
    {
        if ($a['date'] == $b['date']) {
            return 0;
        }
        if ($this->sortMode == 'asc') {
            return ($b['date'] > $a['date']) ? -1 : 1;
        } else {
            return ($a['date'] > $b['date']) ? -1 : 1;
        } 
    }

    function sortByName($a, $b)
    {
        if ($this->sortMode == 'asc') {
            return strcmp(strtolower($a['name']), strtolower($b['name']));
        } else {
            return strcmp(strtolower($b['name']), strtolower($a['name']));
        }
    }

    function sortByTitle($a, $b)
    {
        if ($this->sortMode == 'asc') {
            return strcmp(strtolower($a['title']), strtolower($b['title']));
        } else {
            return strcmp(strtolower($b['title']), strtolower($a['title']));
        }
    }

    function sortByExt($a, $b)
    {
        if ($this->sortMode == 'asc') {
            return strcmp(strtolower($a['ext']), strtolower($b['ext']));
        } else {
            return strcmp(strtolower($b['ext']), strtolower($a['ext']));
        }
    }

    function sortByList($a, $b)
    {
        $key_a = array_search($a['filename'], $this->sortList);
        $key_b = array_search($b['filename'], $this->sortList);
        if ($key_a === false) { return 1; }
        if ($key_b === false) { return -1; }

        if ($key_a > $key_b) {
            return 1;
        } else {
            return -1;
        }
    }


    function sortBySize($a, $b)
    {
        if ($a['size'] == $b['size']) {
            return 0;
        }
        if ($this->sortMode == 'asc') {
            return ($a['size'] < $b['size']) ? -1 : 1;
        } else {
            return ($b['size'] < $a['size']) ? -1 : 1;
        }
    }

    function getAlias($text)
    {
        $str = trim(mb_strtolower($text, 'UTF-8'));

        $accented = array( "á", "é", "í", "ó", "ö", "ő", "ú", "ü", "ű" );  
        $mask     = array( "a", "e", "i", "o", "o", "o", "u", "u", "u" );  
        $str = str_replace( $accented, $mask, $str );  

        $str = str_replace('-', ' ', $str);
        $str = str_replace('.', ' ', $str);
        $str = str_replace('_', ' ', $str);
        $str = preg_replace(array('/\s+/','/[^A-Za-z0-9\-]/'), array('-',''), $str);
        
        return $str;
    }

    private function getDescriptionsInFile()
    {
        if (empty($this->descriptions)) {
            $lines = file( $this->path .DS. $this->descriptionFile);
            
            $items = array();
            for ($i = 0; $i <= count($lines)-1; $i++) {
                $text = trim($lines[$i]);

                // Get the encoding
                $code = array('UTF-8', 'ISO-8859-2');
                $encoding = mb_detect_encoding($text, $code);
                if ($encoding != 'UTF-8') {
                    $text = iconv($encoding, 'UTF-8', $text);
                }

                if (substr($text, 0, 1) == '"') {
                    $text = substr($text, 1);
                    $pos = strpos($text, '"');
                    $id = substr($text, 0, $pos);
                    $title = substr($text, $pos + 2);
                } else {
                    $pos = strpos($text, ' ');
                    $id = substr($text, 0, $pos);
                    $title = substr($text, $pos + 1);
                }
                $title = str_replace('\n', '<br />', $title);
                $title = str_replace('Â', '', $title);
                
                if ($id != '') {
                    $items[$id] = $title;
                }
            }
            $this->descriptions = $items;
        }
    }

    function getTitle($id = null)
    {
        if (isset($this->descriptions[$id]) ) 
        {
            $value = $this->descriptions[$id];
            $value = stripslashes($value);
            $value = str_replace('"', '&quot;', $value);
            return $value;
        } else {
            return $id;
        }
    }

    private function saveDescriptions() 
    {
        if (!empty($this->descriptions)) {
            $content = array();
            foreach ($this->descriptions as $key => $value) {
                $content[] = $key . ' ' . stripslashes($value);
            }
            $content = implode("\r\n", $content);
            file_put_contents($this->path .DS. $this->descriptionFile, $content);
            // chmod($this->path .DS. $this->descriptionFile, 0664);
        }
    }

    function updateDescription($id, $title)
    {
        if (!empty($id)) {
            $this->descriptions[$id] = stripslashes(trim($title));
            $this->saveDescriptions();
        }
    }

    function deleteDescription($id)
    {
        if (!empty($id)) {
            unset($this->descriptions[$id]);
            $this->saveDescriptions();
        }
    }


    function checkFolder($path)
    {
		if ($path != '')
        {
            $paths = explode(DS, $path);
            $tmp = '';
            for ($i = 0; $i <= count($paths)-1; $i++) {
                if ($paths[$i] != '') {
                    $tmp .= DS. $paths[$i];
                    if (!is_dir(DS . $tmp)) {
                        mkdir(DS . $tmp, 0775);
                        file_put_contents(DS . $tmp .DS. 'index.html', '<html><body bgcolor="#FFFFFF"></body></html>');
                        // chmod(DS . $tmp .DS. 'index.html', 0664);
                    }
                }
            }
            return substr($tmp, 1);
        } else {
            return false;
        }
    }

	function filesize_format($bytes, $format = '%01d%s', $force = '')
	{
		$force = strtoupper($force);
		$bytes = max(0, (int)$bytes);

		$units = array('B', 'KB', 'MB', 'GB', 'TB', 'PB');

		$power = array_search($force, $units);

		if ($power === false) {
			$power = $bytes > 0 ? floor(log($bytes, 1024)) : 0;
        }
		return sprintf($format, $bytes / pow(1024, $power), $units[$power]);
	}


}


?>